@@ -1,15 +1,21 @@
 from __future__ import annotations
 
 from abc import ABCMeta, abstractmethod
-from collections.abc import Iterable
-from typing import Any, Callable, Protocol
+from collections.abc import Iterable, Mapping
+from typing import TYPE_CHECKING, Any, NamedTuple, Protocol
 
 from jinja2 import BaseLoader, PackageLoader
-from prompt_toolkit.styles import Style, merge_styles
+from prompt_toolkit.styles import Style
 
-from commitizen import git
-from commitizen.config.base_config import BaseConfig
-from commitizen.defaults import Questions
+from commitizen.exceptions import CommitMessageLengthExceededError
+
+if TYPE_CHECKING:
+    import re
+    from collections.abc import Callable, Iterable, Mapping
+
+    from commitizen import git
+    from commitizen.config.base_config import BaseConfig
+    from commitizen.question import CzQuestion
 
 
 class MessageBuilderHook(Protocol):
@@ -24,6 +30,11 @@ class ChangelogReleaseHook(Protocol):
     ) -> dict[str, Any]: ...
 
 
+class ValidationResult(NamedTuple):
+    is_valid: bool
+    errors: list
+
+
 class BaseCommitizen(metaclass=ABCMeta):
     bump_pattern: str | None = None
     bump_map: dict[str, str] | None = None
@@ -41,7 +52,7 @@ class BaseCommitizen(metaclass=ABCMeta):
         ("disabled", "fg:#858585 italic"),
     ]
 
-    # The whole subject will be parsed as message by default
+    # The whole subject will be parsed as a message by default
     # This allows supporting changelog for any rule system.
     # It can be modified per rule
     commit_parser: str | None = r"(?P<message>.*)"
@@ -68,41 +79,86 @@ class BaseCommitizen(metaclass=ABCMeta):
             self.config.settings.update({"style": BaseCommitizen.default_style_config})
 
     @abstractmethod
-    def questions(self) -> Questions:
+    def questions(self) -> list[CzQuestion]:
         """Questions regarding the commit message."""
 
     @abstractmethod
-    def message(self, answers: dict) -> str:
+    def message(self, answers: Mapping[str, Any]) -> str:
         """Format your git message."""
 
     @property
-    def style(self):
-        return merge_styles(
+    def style(self) -> Style:
+        return Style(
             [
-                Style(BaseCommitizen.default_style_config),
-                Style(self.config.settings["style"]),
+                *BaseCommitizen.default_style_config,
+                *self.config.settings["style"],
             ]
         )
 
+    @abstractmethod
     def example(self) -> str:
         """Example of the commit message."""
-        raise NotImplementedError("Not Implemented yet")
 
+    @abstractmethod
     def schema(self) -> str:
         """Schema definition of the commit message."""
-        raise NotImplementedError("Not Implemented yet")
 
+    @abstractmethod
     def schema_pattern(self) -> str:
         """Regex matching the schema used for message validation."""
-        raise NotImplementedError("Not Implemented yet")
 
+    @abstractmethod
     def info(self) -> str:
         """Information about the standardized commit message."""
-        raise NotImplementedError("Not Implemented yet")
 
-    def process_commit(self, commit: str) -> str:
-        """Process commit for changelog.
+    def validate_commit_message(
+        self,
+        *,
+        commit_msg: str,
+        pattern: re.Pattern[str],
+        allow_abort: bool,
+        allowed_prefixes: list[str],
+        max_msg_length: int | None,
+        commit_hash: str,
+    ) -> ValidationResult:
+        """Validate commit message against the pattern."""
+        if not commit_msg:
+            return ValidationResult(
+                allow_abort, [] if allow_abort else ["commit message is empty"]
+            )
 
-        If not overwritten, it returns the first line of commit.
-        """
-        return commit.split("\n")[0]
+        if any(map(commit_msg.startswith, allowed_prefixes)):
+            return ValidationResult(True, [])
+
+        if max_msg_length is not None and max_msg_length > 0:
+            msg_len = len(commit_msg.partition("\n")[0].strip())
+            if msg_len > max_msg_length:
+                # TODO: capitalize the first letter of the error message for consistency in v5
+                raise CommitMessageLengthExceededError(
+                    f"commit validation: failed!\n"
+                    f"commit message length exceeds the limit.\n"
+                    f'commit "{commit_hash}": "{commit_msg}"\n'
+                    f"message length limit: {max_msg_length} (actual: {msg_len})"
+                )
+
+        return ValidationResult(
+            bool(pattern.match(commit_msg)),
+            [f"pattern: {pattern.pattern}"],
+        )
+
+    def format_exception_message(
+        self, invalid_commits: list[tuple[git.GitCommit, list]]
+    ) -> str:
+        """Format commit errors."""
+        displayed_msgs_content = "\n".join(
+            [
+                f'commit "{commit.rev}": "{commit.message}\n"' + "\n".join(errors)
+                for commit, errors in invalid_commits
+            ]
+        )
+        # TODO: capitalize the first letter of the error message for consistency in v5
+        return (
+            "commit validation: failed!\n"
+            "please enter a commit message in the commitizen format.\n"
+            f"{displayed_msgs_content}"
+        )
