@@ -2,9 +2,9 @@ from __future__ import annotations
 
 import re
 import warnings
-from collections.abc import Sequence
 from dataclasses import dataclass, field
 from functools import cached_property
+from itertools import chain
 from string import Template
 from typing import TYPE_CHECKING, NamedTuple
 
@@ -21,6 +21,7 @@ from commitizen.version_schemes import (
 
 if TYPE_CHECKING:
     import sys
+    from collections.abc import Iterable, Sequence
 
     from commitizen.version_schemes import VersionScheme
 
@@ -88,18 +89,22 @@ class TagRules:
     ignored_tag_formats: Sequence[str] = field(default_factory=list)
     merge_prereleases: bool = False
 
-    @cached_property
-    def version_regexes(self) -> Sequence[re.Pattern]:
-        """Regexes for all legit tag formats, current and legacy"""
-        tag_formats = [self.tag_format, *self.legacy_tag_formats]
-        regexes = (self._format_regex(p) for p in tag_formats)
-        return [re.compile(r) for r in regexes]
+    @property
+    def tag_formats(self) -> Iterable[str]:
+        return chain([self.tag_format], self.legacy_tag_formats)
 
     @cached_property
-    def ignored_regexes(self) -> Sequence[re.Pattern]:
+    def version_regexes(self) -> list[re.Pattern]:
+        """Regexes for all legit tag formats, current and legacy"""
+        return [re.compile(self._format_regex(f)) for f in self.tag_formats]
+
+    @cached_property
+    def ignored_regexes(self) -> list[re.Pattern]:
         """Regexes for known but ignored tag formats"""
-        regexes = (self._format_regex(p, star=True) for p in self.ignored_tag_formats)
-        return [re.compile(r) for r in regexes]
+        return [
+            re.compile(self._format_regex(f, star=True))
+            for f in self.ignored_tag_formats
+        ]
 
     def _format_regex(self, tag_pattern: str, star: bool = False) -> str:
         """
@@ -113,6 +118,10 @@ class TagRules:
             format_regex = format_regex.replace(pattern, regex)
         return format_regex
 
+    def _version_tag_error(self, tag: str) -> str:
+        """Format the error message for an invalid version tag"""
+        return f"Invalid version tag: '{tag}' does not match any configured tag format"
+
     def is_version_tag(self, tag: str | GitTag, warn: bool = False) -> bool:
         """
         True if a given tag is a legit version tag.
@@ -120,9 +129,9 @@ class TagRules:
         if `warn` is `True`, it will print a warning message if the tag is not a version tag.
         """
         tag = tag.name if isinstance(tag, GitTag) else tag
-        is_legit = any(regex.match(tag) for regex in self.version_regexes)
+        is_legit = any(regex.fullmatch(tag) for regex in self.version_regexes)
         if warn and not is_legit and not self.is_ignored_tag(tag):
-            out.warn(f"InvalidVersion {tag} doesn't match any configured tag format")
+            out.warn(self._version_tag_error(tag))
         return is_legit
 
     def is_ignored_tag(self, tag: str | GitTag) -> bool:
@@ -131,8 +140,8 @@ class TagRules:
         return any(regex.match(tag) for regex in self.ignored_regexes)
 
     def get_version_tags(
-        self, tags: Sequence[GitTag], warn: bool = False
-    ) -> Sequence[GitTag]:
+        self, tags: Iterable[GitTag], warn: bool = False
+    ) -> list[GitTag]:
         """Filter in version tags and warn on unexpected tags"""
         return [tag for tag in tags if self.is_version_tag(tag, warn)]
 
@@ -146,9 +155,7 @@ class TagRules:
             m for regex in self.version_regexes if (m := regex.fullmatch(tag.name))
         )
         if not (m := next(candidates, None)):
-            raise InvalidVersion(
-                f"Invalid version tag: '{tag.name}' does not match any configured tag format"
-            )
+            raise InvalidVersion(self._version_tag_error(tag.name))
         if "version" in m.groupdict():
             return self.scheme(m.group("version"))
 
@@ -172,11 +179,7 @@ class TagRules:
             version = self.extract_version(tag)
         except InvalidVersion:
             return False
-
-        if self.merge_prereleases and version.is_prerelease:
-            return False
-
-        return True
+        return not (self.merge_prereleases and version.is_prerelease)
 
     def search_version(self, text: str, last: bool = False) -> VersionTag | None:
         """
@@ -225,7 +228,7 @@ class TagRules:
         version = self.scheme(version) if isinstance(version, str) else version
         tag_format = tag_format or self.tag_format
 
-        major, minor, patch = version.release
+        major, minor, patch = (list(version.release) + [0, 0, 0])[:3]
         prerelease = version.prerelease or ""
 
         t = Template(tag_format)
@@ -238,15 +241,31 @@ class TagRules:
         )
 
     def find_tag_for(
-        self, tags: Sequence[GitTag], version: Version | str
+        self, tags: Iterable[GitTag], version: Version | str
     ) -> GitTag | None:
         """Find the first matching tag for a given version."""
         version = self.scheme(version) if isinstance(version, str) else version
-        possible_tags = [
-            self.normalize_tag(version, f)
-            for f in (self.tag_format, *self.legacy_tag_formats)
-        ]
-        candidates = [t for t in tags if any(t.name == p for p in possible_tags)]
+        release = version.release
+
+        # If the requested version is incomplete (e.g., "1.2"), try to find the latest
+        # matching tag that shares the provided prefix.
+        if len(release) < 3:
+            matching_versions: list[tuple[Version, GitTag]] = []
+            for tag in tags:
+                try:
+                    tag_version = self.extract_version(tag)
+                except InvalidVersion:
+                    continue
+                if tag_version.release[: len(release)] != release:
+                    continue
+                matching_versions.append((tag_version, tag))
+
+            if matching_versions:
+                _, latest_tag = max(matching_versions, key=lambda vt: vt[0])
+                return latest_tag
+
+        possible_tags = set(self.normalize_tag(version, f) for f in self.tag_formats)
+        candidates = [t for t in tags if t.name in possible_tags]
         if len(candidates) > 1:
             warnings.warn(
                 UserWarning(
